<?php

namespace Drupal\pf_email\Plugin\PushFrameworkChannel;

use Drupal;
use Drupal\Component\Render\PlainTextOutput;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\Core\Mail\MailManagerInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\push_framework\ChannelBase;
use Drupal\user\UserInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the push framework channel.
 *
 * @ChannelPlugin(
 *   id = "email",
 *   label = @Translation("Email"),
 *   description = @Translation("Provides the email channel plugin.")
 * )
 */
class Email extends ChannelBase {

  /**
   * @var bool
   */
  protected $active;

  /**
   * @var \Drupal\Core\Mail\MailManagerInterface
   */
  protected $mailManager;

  /**
   * @inheritDoc
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, ConfigFactoryInterface $config_factory, EntityTypeManagerInterface $entity_type_manager, LoggerChannelInterface $logger, RendererInterface $renderer, MailManagerInterface $mail_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $config_factory, $entity_type_manager, $logger, $renderer);
    $this->mailManager = $mail_manager;
  }

  /**
   * @inheritDoc
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('config.factory'),
      $container->get('entity_type.manager'),
      $container->get('logger.channel.push_framework'),
      $container->get('renderer'),
      $container->get('plugin.manager.mail')
    );
  }

  /**
   * @inheritDoc
   */
  public function init(): void {
    $this->active = TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function applicable(UserInterface $user): bool {
    return $this->active;
  }

  /**
   * {@inheritdoc}
   */
  public function send(UserInterface $user, ContentEntityInterface $entity, $attempt): string {
    if (!$this->active) {
      return self::RESULT_STATUS_FAILED;
    }
    $message = $this->mailManager->mail('pf_email', 'notification', $user->getEmail(), $user->getPreferredLangcode(), [
      'subject' => $entity->label(),
      'body' => $this->render($entity, $user->getPreferredLangcode()),
    ]);

    return $message['result'] ? self::RESULT_STATUS_SUCCESS : self::RESULT_STATUS_FAILED;
  }

  protected function render(ContentEntityInterface $entity, $langcode) {
    $viewBuilder = $this->entityTypeManager->getViewBuilder($entity->getEntityTypeId());
    $elements = $viewBuilder->view($entity, 'teaser', $langcode);
    return trim(PlainTextOutput::renderFromHtml($this->renderer->renderPlain($elements)));
  }

}
